@@ -41,7 +41,7 @@ namespace System.Data
 		IExpression rowFilterExpr;
 		string sort = String.Empty;
 		protected DataViewRowState rowState;
-		protected DataRowView[] rowCache = null;
+		protected DataRowView[] rowCache = new DataRowView [0];
 
 		// BeginInit() support
 		bool isInitPhase = false;
@@ -226,8 +226,10 @@ namespace System.Data
 					rowFilterExpr = parser.Compile (value);
 				}
 				rowFilter = value;
-				if (!inEndInit)
+				if (!inEndInit) {
 					UpdateIndex (true);
+					OnListChanged(DataView.ListResetEventArgs);
+				}
 			}
 		}
 
@@ -246,8 +248,10 @@ namespace System.Data
 					return;
 
 				rowState = value;
-				if (!inEndInit)
+				if (!inEndInit) {
 					UpdateIndex (true);
+					OnListChanged(DataView.ListResetEventArgs);
+				}
 			}
 		}
 
@@ -255,7 +259,12 @@ namespace System.Data
 		[DataSysDescription ("Indicates the order in which data is returned by this DataView.")]
 		[DefaultValue ("")]
 		public string Sort {
-			get { return sort; }
+			get { 
+				if (useDefaultSort)
+					return String.Empty;
+				else
+					return sort;
+			}
 			set {
 				if (isInitPhase) {
 					initSort = value;
@@ -264,7 +273,7 @@ namespace System.Data
 				if (value == sort)
 					return;
 
-				if (value == null) {	
+				if ((value == null) || (value.Equals (String.Empty))) {
 				/* if given value is null useDefaultSort */
 					useDefaultSort = true;
 					/* if ApplyDefault sort is true try appling it */
@@ -278,8 +287,10 @@ namespace System.Data
 					sort = value;
 					//sortedColumns = SortableColumn.ParseSortString (dataTable, value, true);
 				}
-				if (!inEndInit)
+				if (!inEndInit) {
 					UpdateIndex (true);
+					OnListChanged(DataView.ListResetEventArgs);
+				}
 			}
 		}
 
@@ -311,8 +322,10 @@ namespace System.Data
 					sort = null;
 					rowFilter = null;
 					rowFilterExpr = null;
-					if (!inEndInit)
+					if (!inEndInit) {
 						UpdateIndex (true);
+						OnListChanged(DataView.ListResetEventArgs);
+					}
 				}
 			}
 		}
@@ -535,7 +548,7 @@ namespace System.Data
 			if (dataTable != null)
 				UnregisterEventHandlers ();
 			Index = null;
-			rowCache = null;
+			rowCache = new DataRowView [0];
 			isOpen = false;
 		}
 
@@ -703,9 +716,9 @@ namespace System.Data
 		{
 			// TODO: what really happens?
 			Close ();
-			rowCache = null;
+			rowCache = new DataRowView [0];
 			Open ();
-			OnListChanged (new ListChangedEventArgs (ListChangedType.Reset, -1 ));
+			OnListChanged (new ListChangedEventArgs (ListChangedType.Reset, -1, -1 ));
 		}
 
 #if NET_2_0
@@ -758,8 +771,6 @@ namespace System.Data
 			else {
 				rowCache = new DataRowView[0];
 			}
-
-			OnListChanged (new ListChangedEventArgs (ListChangedType.Reset, -1, -1));
 		}
 
 		internal virtual IExpression FilterExpression
