@@ -4,7 +4,30 @@
 // Author:
 //	Atsushi Enomoto <atsushi@ximian.com>
 //
-// (C)2004 Novell Inc.
+
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 
 using System;
 using System.Collections;
@@ -475,5 +498,35 @@ namespace MonoTests.System.Data
 			// a:foo is ignored
 			AssertDataTable ("dt", ds.Tables [0], "root", 1, 0, 0, 0, 0, 0);
 		}
+
+		[Test]
+		public void ContainsSchema ()
+		{
+			DataSet ds = new DataSet();
+			DataTable dt1 = new DataTable();
+			ds.Tables.Add(dt1);
+			DataColumn dc1 = new DataColumn("Col1");
+			dt1.Columns.Add(dc1);
+			dt1.Rows.Add(new string[] { "aaa" } );
+			DataTable dt2 = new DataTable();
+			ds.Tables.Add(dt2);
+			DataColumn dc2 = new DataColumn("Col2");
+			dt2.Columns.Add(dc2);
+			dt2.Rows.Add(new string[] { "bbb" } );
+
+			DataRelation rel = new DataRelation("Rel1",dc1,dc2,false);
+			ds.Relations.Add(rel);
+
+			StringWriter sw = new StringWriter ();
+			ds.WriteXml(sw, XmlWriteMode.WriteSchema);
+
+			ds = new DataSet();
+			ds.ReadXml(new StringReader (sw.ToString ()));
+			sw = new StringWriter ();
+			ds.WriteXml(sw);
+			XmlDocument doc = new XmlDocument ();
+			doc.LoadXml (sw.ToString ());
+			AssertEquals (2, doc.DocumentElement.ChildNodes.Count);
+		}
 	}
 }
