@@ -79,12 +79,10 @@ namespace System.Web
 
 		public void Add (string key, string value)
 		{
-			lock (this) {
-				if (data == null)
-					data = new ListDictionary ();
+			if (data == null)
+				data = new ListDictionary ();
 
-				data.Add (key, value);
-			}
+			data.Add (key, value);
 		}
 
 		public Hashtable GetProperties (Hashtable tbl)
@@ -131,10 +129,10 @@ namespace System.Web
 				expression = expression.Substring (text.Length);
 			}
 			
-			lock (this) {
-				if (pattern == null)
-					return expression.Length == 0;
+			if (pattern == null)
+				return expression.Length == 0;
 
+			lock (this) {
 				if (regex == null)
 					regex = new Regex (pattern);
 			}
@@ -145,11 +143,19 @@ namespace System.Web
 	
 	class CapabilitiesLoader : MarshalByRefObject
 	{
-		static bool loaded;
+		static volatile bool loaded;
 		static ICollection alldata;
 		static Hashtable defaultCaps;
+		static readonly object lockobj = new object ();
 		private CapabilitiesLoader () {}
 
+		static CapabilitiesLoader ()
+		{
+			defaultCaps = new Hashtable ();
+			defaultCaps.Add ("frames", "True");
+			defaultCaps.Add ("tables", "True");
+		}
+			
 		public static Hashtable GetCapabilities (string userAgent)
 		{
 			Init ();
@@ -157,39 +163,74 @@ namespace System.Web
 				userAgent = userAgent.Trim ();
 
 			if (alldata == null || userAgent == null || userAgent == "")
-				return DefaultCapabilities;
+				return defaultCaps;
 
 			foreach (BrowserData bd in alldata) {
-				if (bd.IsMatch (userAgent)) {
+				if (bd.IsMatch (userAgent))
 					return bd.GetProperties (new Hashtable ());
-				}
 			}
 			
-			return DefaultCapabilities;
+			return defaultCaps;
 		}
 
 		static void Init ()
 		{
-			lock (typeof (CapabilitiesLoader)) {
+			if (loaded)
+				return;
+
+			lock (lockobj) {
 				if (loaded)
 					return;
-
-				string path = Path.GetDirectoryName (WebConfigurationSettings.MachineConfigPath);
-				path = Path.Combine (path, "browscap.ini");
+#if TARGET_J2EE
+				string filepath = "browscap.ini"
+#else
+				string dir = Path.GetDirectoryName (WebConfigurationSettings.MachineConfigPath);
+				string filepath = Path.Combine (dir, "browscap.ini");
+				if (!File.Exists (filepath)) {
+					// try removing the trailing version directory
+					dir = Path.GetDirectoryName (dir);
+					filepath = Path.Combine (dir, "browscap.ini");
+				}
+#endif
 				try {
-					LoadFile (path);
+					LoadFile (filepath);
 				} catch (Exception) { }
 
 				loaded = true;
 			}
 		}
-
+#if TARGET_J2EE
+	private static TextReader GetJavaTextReader(string filename)
+	{
+		Stream s;
+		try
+		{
+			java.lang.ClassLoader cl = (java.lang.ClassLoader)
+						AppDomain.CurrentDomain.GetData("GH_ContextClassLoader");
+			if (cl == null)
+				return null;
+			java.io.InputStream inputStream = cl.getResourceAsStream(filename);
+			s = (Stream)vmw.common.IOUtils.getStream(inputStream);
+		}
+		catch (Exception e)
+		{
+			return null;
+		}
+		return = new StreamReader (s);
+	}
+#endif
 		static void LoadFile (string filename)
 		{
+#if TARGET_J2EE
+			TextReader input = GetJavaTextReader(filename);
+			if(input == null)
+				return;
+#else
 			if (!File.Exists (filename))
 				return;
 
 			TextReader input = new StreamReader (File.OpenRead (filename));
+#endif
 			string str;
 			Hashtable allhash = new Hashtable ();
 			int aux = 0;
@@ -233,20 +274,6 @@ namespace System.Web
 				data.Add (keyvalue [0], keyvalue [1]);
 			}
 		}
-
-		static Hashtable DefaultCapabilities {
-			get {
-				lock (typeof (CapabilitiesLoader)) {
-					if (defaultCaps != null)
-						return defaultCaps;
-
-					defaultCaps = new Hashtable ();
-					defaultCaps.Add ("frames", "True");
-					defaultCaps.Add ("tables", "True");
-					return defaultCaps;
-				}
-			}
-		}
 	}
 }
 
